# Copyright (c) 2020  University of Houston

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import logging
import logging.handlers
import os
import sys

from flask import Flask, request, request_started

from .config import LISTENPORT, LOGLEVEL, LOGFORMAT, GRADER_LOG
from .grader import grade
from .json import jsonify

def create_app():
  app = Flask(__name__)

  lhandle = logging.handlers.RotatingFileHandler(GRADER_LOG, maxBytes=100000, backupCount=10)
  lhandle.setLevel(LOGLEVEL)
  lhandle.setFormatter(logging.Formatter(LOGFORMAT))
  app.logger.addHandler(lhandle)
  app.logger.setLevel(LOGLEVEL)

  @app.route('/', methods=["GET", "POST"])
  def root():
    try:
      if request.method == 'GET':
        return "Print Usage"

      if not request.json:
        return jsonify(None, -1, "Missing json POST data")
      req = request.json

      if 'data' not in req:
        return jsonify(None, -1, "Missing json POST data")

      data = req['data']
      answerkey = req['answerkey']
      attempt = req['attempt']
      res = grade(data, answerkey, attempt)

      return jsonify(res)
    except Exception as e:
      app.logger.exception("Unhandled exception")
      return jsonify(e)

  return app

