# Copyright (c) 2019  University of Houston

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

import json

from logging import StreamHandler

try:
  import czlog.config
  import czlog.util
  ACTIVE = True
except ImportError:
  ACTIVE = False


class CZLogHandler(StreamHandler):
  def __init__(self, topic):
    super(CZLogHandler, self).__init__()
    self.topic = topic
    self.__socket = None

  @property
  def _socket(self):
    if not self.__socket:
      self.__socket = czlog.util.bindPublisher()
    return self.__socket

  def emit(self, record):
    if not ACTIVE:
      return
    msg = json.dumps({
        "method": "client-log",
        "client": self.topic,
        "message": record.msg,
        "level": record.levelno
    })
    self._socket.send(msg)


class _Logger(object):
  def __init__(self, topic):
    self._socket = None
    self.topic = None

  @property
  def socket(self):
    if not self._socket:
      self._socket = czlog.util.bindPublisher()
    return self._socket

  def log(self, message):
    if not ACTIVE:
      return
    msg = json.dumps({
        "method": "client-log",
        "client": self.topic,
        "message": message
    })
    self.socket.send(msg)


Logger = _Logger('grader')
