#!/usr/bin/env python

from __future__ import absolute_import

import argparse
import datetime
import json
import os

from grader import client

def parseArgs():
  parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
  parser.add_argument("--due-date", dest="due_date", type=str)
  parser.add_argument("--stunames", dest="stunamespath", type=str, default=None)
  parser.add_argument("--filepath", dest="filepath", type=str, default=".")
  return parser.parse_args()


if __name__ == "__main__":
  opts = parseArgs()

  due_date = datetime.datetime.strptime(opts.due_date, '%Y%m%d%H%M')
  
  scores = {}
  
  for f_name in os.listdir(opts.filepath):
    fname = "%s/%s" % (opts.filepath, f_name)
    score = 0.
    try:
      student = fname.split('+')[3]
    except IndexError as e:
      continue
    student = student.split('-')[0]
    
    attempt = fname.split('.')[-2:]
    attempt = int(attempt[0])+ 0.000001 * int(attempt[1])
    attempt = datetime.datetime.fromtimestamp(attempt)
  
    with open(fname) as fhandler:
      data = json.load(fhandler)
      total = data.pop('total')
      success = data.pop('success')
    score = success/total
  
    if attempt < due_date:
      scores.setdefault(student, [])
      scores[student].append((attempt, score))
  
  if opts.stunamespath:
    names = client.loadUsernameCSV(opts.stunamespath)
    f, l, combined_padding, uname_padding = client.namePadding(names)
    missing = scores.keys()

    print_array = []
    for username in names.keys():
      try:
        student = username
        attempts = scores[username]
        date, attempt = sorted(attempts, key=lambda x: x[0], reverse=True)[0]
        if date > due_date:
          attempt = str(attempt) + " ***LATE***"
      except KeyError:
        attempt = "No attempt found"

      try:
        first, last = (names[username]["first"], names[username]["last"])
      except KeyError:
        first, last = "", "Name not found"
        
      print_array.append("%s -- %s -- %s" % (str.ljust(last + ", " + first, combined_padding + 2),
                                                 str.ljust(str(username), uname_padding),
                                                 str(attempt)))

    for user in missing:
      if user not in names.keys():
        print("Missing name: %s" % user)

    for line in sorted(print_array):
      print(line)

  else:
    for student, attempts in scores.items():
      print(student, sorted(attempts, key=lambda x: x[0], reverse=True)[0])

