# Copyright (c) 2020  University of Houston
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/

import dpkt
import six

from uhnet.pcap.eth import parse as eth_parse
from uhnet.pcap.arp import parse as arp_parse

from .actiongroups import hostPort2ip
from .base import BasePacket

class ARPReplyPacket(BasePacket):
  def __init__(self, host, interface, pcaps):
    self.hostname = host
    self._intf = interface
    self._pcaps = pcaps
    self.headers = None

  def __repr__ (self):
    return("{0}: {1}, {2}, {3}".format(self.__class__.__name__, 
                           str(self.hostname), str(self._intf), str(self._pcaps)))

  def resolve(self, data):
    # We can only take 1 pcap file
    name = self._pcaps[0]
    pcap_meta = data['pcap-roster'][name]
    pcap_name = pcap_meta['pcap-name']
    mac = data['manifest'][self.hostname][self._intf].macaddress
    ip = hostPort2ip(data, (self.hostname, self._intf))
    ip = ip.split('/')[0]
    # only one packet?

    fl = six.BytesIO(data[pcap_name])

    reader = dpkt.pcap.Reader(fl)
    for (ts, pkt) in reader:
      eth = eth_parse(pkt)
      arp = arp_parse(eth)
      if str(arp['tpa']) != ip:
        continue
      self.headers = [
          {'Source MAC': str(mac)},
          {'Destination MAC': str(arp['src'])},
          {'Ethertype': '0x0806'},
          {'ARP Opcode': '2'},
          {'ARP SHA': str(mac)},
          {'ARP SPA': str(arp['tpa'])},
          {'ARP THA': str(arp['sha'])},
          {'ARP TPA': str(arp['spa'])}
      ]

  def cleanAnswer (self, other):
    for idx,val in enumerate(other):
      newd = {}
      for k,v in val.items():
        newd[k] = v.strip()
      other[idx] = newd

  def check(self, other):
    self.cleanAnswer(other)

    value = 0.
    for hdr in self.headers:
      if hdr in other:
        value += 1
    return value/len(self.headers)
