# Copyright (c) 2019-2020  University of Houston

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

import getpass
import json

try:
  import czlog.config
  import czlog.util
  ACTIVE = True
except ImportError:
  ACTIVE = False


class _Logger(object):
  def __init__ (self):
    self._socket = None
    self._user = None
    if not ACTIVE:
      self.log = self._log_dummy

  @property
  def socket (self):
    if not self._socket:
      self._socket = czlog.util.bindPublisher()
      self._user = getpass.getuser()
    return self._socket

  @property
  def user (self):
    if not self._user:
      self._socket = czlog.util.bindPublisher()
      self._user = getpass.getuser()
    return self._user

  def log (self, message):
    self.socket.send_json({"method" : "client-log", "client" : "user-%s" % (self.user),
                                 "message" : message})

  def _log_dummy (self, message):
    return


Logger = _Logger()
