# Copyright (c) 2021  University of Houston

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import ipywidgets as widgets
from ..widgetutils import Layouts

class NETBrowser(object):
  def __init__ (self, net):
    self._net = net
    self._dmap = {}
    self._buildData()

    self._w_framebox = None
    self._w_ddhbox = None
    self._w_output = widgets.Output(layout = {"border" : "1px solid black"})

    dummy = ["  -"]
    dummy.extend(self._dmap.keys())

    self._w_vdd = widgets.Dropdown(options = dummy, layout = Layouts.AUTOWIDTH)
    self._w_ctr_funcs = widgets.Dropdown(options = ["  -"], layout = Layouts.AUTOWIDTH)

    self._w_vdd.observe(self._ctrchoice)
    self._w_ctr_funcs.observe(self._funcchoice)

  def _buildData (self):
    for ctr in self._net._containers.values():
      self._dmap[ctr.client_id] = {}
      for name in dir(ctr):
        m = getattr(ctr, name)
        if hasattr(m, "uiname"):
          self._dmap[ctr.client_id][m.uiname] = m

  def _ctrchoice (self, change):
    if change["type"] == 'change' and change["name"] == "value":
      if change["new"] == "  -":
        self._w_ctr_funcs.options = ["  -"]
      else:
        d = ["  -"]
        d.extend(sorted(self._dmap[change["new"]].keys()))
        self._w_ctr_funcs.options = d
      self._w_ctr_funcs.value = "  -"

  def _funcchoice (self, change):
    if change["type"] == 'change' and change["name"] == "value":
      if change["new"] == "  -":
        return
      ctr = self._w_vdd.value
      func = self._dmap[ctr][self._w_ctr_funcs.value]
      out = func()
      self._w_output.clear_output()

      with self._w_output:
        self._w_output.append_display_data(out)

  def display (self):
      if not self._w_framebox:
          self._w_ddhbox = widgets.HBox([self._w_vdd, self._w_ctr_funcs])
      self._w_framebox = widgets.VBox([self._w_ddhbox, self._w_output])
      return self._w_framebox
