# Copyright (c) 2021  University of Houston

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import getpass

import ipywidgets as widgets
from ..widgetutils import Layouts

from . import browser
from . import NetworkWrapper
from .. import creds
from .. import util

class UserDashboard(object):
  def __init__ (self, uhm, width=3):
    self._uhm = uhm
    self._width = width
    self._uid = None
    self._uurn = None
    self._net = None

    self._usermap = {urn.split("+")[-1]:urn for urn in self._uhm._lsp.users}
    dummy = ["  -"]
    dummy.extend(self._usermap.keys())
    dummy.sort()

    self._w_user_dd = widgets.Dropdown(description = "User:", options = dummy,
                                       layout = Layouts.AUTOWIDTH)
    self._w_slice_input = widgets.Text(description = "Slice:")
    self._w_show_button = widgets.Button(description = "Show")
    self._w_show_button.on_click(self.showbtn)
    self._w_graph = None
    self._w_vbrowse = widgets.VBox()

    self._w_ihbox = widgets.HBox([self._w_user_dd, self._w_slice_input, self._w_show_button])
    self._w_framevbox = widgets.VBox([self._w_ihbox])

  def showbtn (self, button):
    # Reset while we're doing work
    self._w_framevbox.children = [self._w_ihbox]

    self._uid = self._w_user_dd.value
    self._uurn = self._usermap[self._w_user_dd.value]
    sname = "%s-%s" % (self._uid, self._w_slice_input.value)
    if not util.userHasSliceCred(self._uhm.context, sname):
      user_cert = creds.getUserCert(self._uhm._uid)
      sdata = {}
      sdata[self._uhm._course] = { self._uhm._uid : [sname, user_cert]}
      util.issueUserSliceCred(self._uhm._course, sdata)
    site = util.getUserSite(self._uhm._course, self._uurn)
    sinfo = self._uhm.getSliceInfo(sname, site)
    self._net = NetworkWrapper(self._uhm, sinfo.manifest, sinfo.name, None)
    self._net.site = site

    graph = widgets.Output()
    with graph:
      graph.append_display_data(self._net.showtopo())
    intfs = widgets.Output()
    with intfs:
      intfs.append_display_data(self._net.ShowInterfaces())
    over_hbox = widgets.HBox([graph, intfs], layout = widgets.Layout(width="99%"))
    
    bl = []
    for x in range(0, self._width):
      bl.append(browser.NETBrowser(self._net))
    bwl = [b.display() for b in bl]
    browser_hbox = widgets.HBox(bwl)

    self._w_framevbox.children = [self._w_ihbox, over_hbox, browser_hbox]


  def display(self):
    return self._w_framevbox
