# Copyright (c) 2018  University of Houston Networking Lab

from __future__ import absolute_import

class UnknownEdgeVertexerror(Exception):
  def __init__ (self, vertex):
    self.v = vertex
  def __str__ (self):
    return "Vertex %s is not connected to this edge" % (str(self.v))

class GraphAlreadyAttachedError(Exception):
  def __str__ (self):
    return "This object is already a member of a graph"


class GraphBase(object):
  def __init__ (self):
    self._edges = {}
    self._vertices = {} # { <id> : <vertex-obj>, ... }

  def addEdge (self, edge):
    edge._graphAttach(self)
    self._edges[edge._id] = edge
    return edge

  def removeEdges (self, edges):
    for edge in edges:
      edge._graph = None
      del self._edges[edge._id]

  def addVertex (self, vertex):
    vertex._graphAttach(self)
    self._vertices[vertex._id] = vertex
    return vertex

  def removeVertex (self, vertex):
    vertex._graph = None
    del self._vertices[vertex._id]

  def getDotString (self):
    dot_data = []
    dda = dot_data.append
    dda("graph {")

    for edge in self._edges.values():
      dda(""""%d" -- "%d" """ % (edge._vertex_a._id, edge._vertex_b._id))

    for vertex in self._vertices.values():
      dda(""""%d" [label="%s",shape=%s]""" % (vertex._id, vertex.name, vertex.dotshape))

    dda("}")
    return "\n".join(dot_data)


class VertexBase(object):
  ID = 0

  def __init__ (self):
    self._id = VertexBase.ID
    VertexBase.ID += 1
    self.dotshape = "oval"

    self._graph = None
    self._edges = {}
    self._name = None

  @property
  def name (self):
    return self._name

  def __str__ (self):
    return "<%d>" % (self._id)

  def _graphAttach (self, graph):
    if self._graph:
      raise GraphAlreadyAttachedError()
    self._graph = graph

  def addEdge (self, edge):
    self._edges[edge._id] = edge

  def getEdge (self, other_vertex):
    for edge in self._edges.values():
      if edge.traverse(self) == other_vertex:
        return edge

  def copyUnconnected (self):
    newv = self.__class__()
    newv._name = self._name
    newv._id = self._id
    return newv


class EdgeBase(object):
  ID = 0

  def __init__ (self):
    self._id = EdgeBase.ID
    EdgeBase.ID += 1
    self._graph = None
    self._vertex_a = None
    self._vertex_b = None

  def _graphAttach (self, graph):
    if self._graph:
      raise GraphAlreadyAttachedError()
    self._graph = graph

  def traverse (self, vertex):
    if vertex == self._vertex_a:
      return self._vertex_b
    elif vertex == self._vertex_b:
      return self._vertex_a
    else:
      raise UnknownEdgeVertexError(vertex)

