# Copyright (c) 2017-2018  University of Houston

from __future__ import absolute_import

from . import core as G
from . import layer2 as L2

import geni.aggregate.vts as VTSAM

def buildFromManifest(manifest, name = None, site = None):
  g = G.Graph()
  g.name = name
  if site:
    g.site = site
  else:
    g.site = VTSAM.aggregateFromHost(manifest.host)
  g.slicename = manifest.slicename.split("+")[-1]

  for dp in manifest.datapaths:
    g.addVertex(G.L2Datapath(dp.client_id))

  for cntr in manifest.containers:
    cv = _containerBuilder(cntr)
    g.addVertex(cv)

  bidi = set()
  for port in manifest.ports:
    kl = [port.client_id, port.remote_client_id]
    kstr = "".join(sorted(kl))
    if kstr in bidi:
      continue
    g.addEdge(g[port.dpname], g[port.remote_dpname])
    bidi.add(kstr)

  g.applyManifest(manifest)
  return g


def _containerBuilder (container):
  if container.image == "uh.quagga":
    return G.QuaggaContainer(container.client_id)
  elif container.image == "uh.net-client":
    return G.AlpineContainer(container.client_id)
  else:
    return G.ContainerVertex(container.client_id)


def buildL2FromManifest (manifest):
  g = L2.L2Graph()

  ports = {}
  for dp in manifest.datapaths:
    dpv = g.addVertex(L2.Node(dp.client_id))

    for port in dp.ports:
      portv = dpv.addPort(L2.Port(port.name))
      portv.client_id = port.client_id
      ports[port.client_id] = portv

  for ctr in manifest.containers:
    cv = g.addVertex(L2.Node(ctr.client_id))

    for port in ctr.ports:
      portv = cv.addPort(L2.EthernetPort(port.name, port.macaddress))
      portv.client_id = port.client_id
      ports[port.client_id] = portv

  for vf in manifest.functions:
    fv = g.addVertex(L2.Node(vf.client_id))
    ports[vf.client_id] = fv

  for port in manifest.ports:
    pv = ports[port.client_id]
    try:
      rpv = ports[port.remote_client_id]
    except KeyError:
      rpv = g.addVertex(L2.Node(port.remote_client_id))

    if not pv.getEdge(rpv):
      edge = L2.LinkEdge(pv, rpv)
      g.addEdge(edge)

  return g

# Too specific to certain topologies for now (how to identify bridges?)

#def generateMACFlows (tree, node_name, startmac = None, extra_match = None):
#  if extra_match is None:
#    extra_match = {}
#
#  startv = tree[node_name]
#  flows = []
#
#  visit_list = []
#  for port in startv.portIter():
#    if not startmac:
#      startmac = port._macaddr
#    rport = port.getLinkedPort()
#    visit_list.append((rport, rport.getNode()))
#
#  iteration = 0
#  while visit_list:
#    iteration += 1
#    if iteration > 50:
#      raise ValueError()
#    (port, node) = visit_list.pop()
#    match = {"dl_dst" : startmac, "priority" : 2000}
#    match.update(extra_match)
#    flows.append({"datapath" : node.name, "output" : port.client_id, "match" : match})
#
#    for outport in node.portIter():
#      if outport == port:
#        continue
#      rport = outport.getLinkedPort()
#      rnode = rport.getNode()
#      if rnode.name.count("bridge"):
#        visit_list.append((rport, rnode))
#
#  return flows
#
#def generateBroadcastFlows (tree):
#  flows = []
#  match = {"dl_dst" : "ff:ff:ff:ff:ff:ff", "priority" : 1000}
#  for node in tree._nodes_byname.values():
#    if not node.name.count("bridge"):
#      continue
#    outlist = [x.client_id for x in node.portIter()]
#    flows.append({"datapath" : node.name, "output" : outlist, "match" : match})
#  return flows
