# Copyright (c) 2018  University of Houston

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

from bqplot import Figure, LinearScale # pylint: disable=F0401
from bqplot.marks import Graph # pylint: disable=F0401
import geni.rspec.vtsmanifest as VTSM
import geni.util
import pydot

X_SHIFT = -10
Y_MINOFFSET = 20
Y_MAXOFFSET = 0

def getGraphvizPositions (manifest, engine):
  g = pydot.graph_from_dot_data(geni.util.builddot([manifest]))
  newg = pydot.graph_from_dot_data(g[0].create_dot(prog=engine))

  posmap = {}
  for node in newg[0].get_nodes():
    if node.get_pos() is None:
      continue
    if node.get_name()[0] == '"':
      name = node.get_name()[1:-1]
    else:
      name = node.get_name()
    (x,y) = node.get_pos()[1:-1].split(",")
    posmap[name] = (float(x) + X_SHIFT, float(y))

  for subg in newg[0].get_subgraphs():
    for node in subg.get_nodes():
      if node.get_pos() is None:
        continue
      if node.get_name()[0] == '"':
        name = node.get_name()[1:-1]
      else:
        name = node.get_name()
      (x,y) = node.get_pos()[1:-1].split(",")
      posmap[name] = (float(x) + X_SHIFT, float(y))

  return posmap


def buildBQWidget (posmap, data):
  manifest = data["manifest"]
  nodes = []
  for dp in manifest.datapaths:
    nodes.append({"label" : dp.client_id, "shape" : "ellipse",
                  "shape_attrs" : {'ry' : 15, 'rx' : 35}, "label_display" : "center"})
  for ctr in manifest.containers:
    nodes.append({"label" : ctr.client_id, "shape" : "rect",
                  "shape_attrs" : {'height' : 30, 'width' : 70}, "label_display" : "center"})

  node_index_map = {node["label"]: i for i,node in enumerate(nodes)}

  edges = []
  for port in manifest.ports:
    if isinstance(port, VTSM.InternalPort):
      edges.append({"source" : node_index_map[port.dpname], "target" : node_index_map[port.remote_dpname]})
    elif isinstance(port, VTSM.InternalContainerPort):
      edges.append({"source" : node_index_map[port.dpname], "target" : node_index_map[port.remote_dpname]})

  x = []
  y = []
  colors = []
  for node in nodes:
    (xp, yp) = posmap[node["label"]]
    x.append(xp)
    y.append(yp)
    colors.append("#FFFFFF")

  xs = LinearScale(min=0, max=max(x)+20)
  ys = LinearScale(min=0+Y_MINOFFSET, max=max(y)+Y_MAXOFFSET)

  graph = Graph(node_data = nodes, link_data = edges,
                scales={'x' : xs, 'y' : ys }, x = x, y = y,
                link_type = "line", directed = False,
                colors = colors)

  layout = data["_wlayout"]
#  layout["width"] = "%dpx" % (int(max(x) * 1.9))
  graphf = Figure(marks=[graph], fig_margin = {"top" : 1, "bottom" : 1, "left" : 1, "right" : 10}, layout = layout)
#                  layout = widgets.Layout(height='%dpx' % (int(max(y) * 1.2)), width='%dpx' % (int(max(x) * 1.9))))

  return graphf
