# Copyright (c) 2019  University of Houston Networking Lab

from __future__ import absolute_import

import ipywidgets as widgets # pylint: disable=F0401

class PacketFieldManager(object):
  XRCSFMAP = {
    "src-mac" : ("Source MAC", None),
    "dst-mac" : ("Destination MAC", None),
    "ether-type" : ("Ethertype", None),
    "arp-op" : ("ARP Opcode", None),
    "arp-sha" : ("ARP SHA", None),
    "arp-spa" : ("ARP SPA", None),
    "arp-tha" : ("ARP THA", None),
    "arp-tpa" : ("ARP TPA", None)
  }

  def __init__ (self, data, opts):
    self._data = data
    self._opts = opts
    self.dpindex = data["index"]
    self._w_vboxl = None
    self._w_vboxi = None
    self._w_hbox = None

    self._build()

  def _build (self):
    labels = []
    inputs = []
    for field in self._opts["fields"]:
      (fname, typ) = PacketFieldManager.XRCSFMAP[field]
      labels.append(widgets.Label(value = fname, layout = widgets.Layout(width="auto")))
      inputs.append(widgets.Text())
    self._w_vboxl = widgets.VBox(children = labels, layout = widgets.Layout(width="auto"))
    self._w_vboxi = widgets.VBox(children = inputs)
    self._w_hbox = widgets.HBox([self._w_vboxl, self._w_vboxi], layout = widgets.Layout(width="80%"))

  def display (self):
    return self._w_hbox

  def getAnswerData (self):
    data = [{"_uidata" : self._uidata}]
    for lbl,inp in zip(self._w_vboxl.children, self._w_vboxi.children):
      obj = { lbl.value : inp.value }
      data.append(obj)
    return data
