# Copyright (c) 2018  University of Houston Networking Lab

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

from IPython.display import display
import ipywidgets as widgets

class ProgressStatusBar(object):
  def __init__ (self, min, max, desc, status, step = 1, value = 1):
    self.bar_style = "info"
    self.value = value
    self.min = min
    self.max = max
    self.desc = desc
    self.step = step
    self.start_status = status
    
    self._w_pbar = widgets.IntProgress(self.value, self.min, self.max, self.step, bar_style = self.bar_style,
                                       description = self.desc, orientation = "horizontal")
    self._w_sbar = widgets.Text(self.start_status, layout = widgets.Layout(width="96%"))
    self._w_pbox = widgets.VBox([self._w_pbar, self._w_sbar], layout = widgets.Layout(width="100%"), width='100%')

  def display (self):
    display(self._w_pbox)

  def reset (self):
    self._w_pbar.bar_style = "info"
    self._w_pbar.value = 1
    self._w_sbar.value = self.start_status

  def fail (self, status):
    self._w_sbar.value = status
    self._w_pbar.bar_style = "danger"

  def success (self, status):
    self._w_sbar.value = status
    self._w_pbar.value = self._w_pbar.max
    self._w_pbar.bar_style = "success"

  def setStatus (self, status, inc = False):
    self._w_sbar.value = status
    if inc:
      self.incrementProgress()

  def incrementProgress (self):
    self._w_pbar.value = self._w_pbar.value + 1
