# Copyright (c) 2018  University of Houston Networking Lab

from __future__ import absolute_import

import ipywidgets as widgets # pylint: disable=F0401

from .. import inputtable as IT
from ..widgetutils import getInterfaces

class SubnetTabManager(object):
  def __init__ (self, data):
    self._data = data
    self.dpindex = data["index"]
    self._wtab = widgets.Tab()

    self._wbutton = None
    if not ("suppress-button" in self._data) or (not self._data["suppress-button"]):
      if "button-text" in self._data:
        self._wbutton = widgets.Button(description = self._data["button-text"], icon = "plus")
      else:
        self._wbutton = widgets.Button(description = "Add Subnet", icon = "plus")

    if self._wbutton:
      self._wbutton.on_click(self.addTab)
      self._vbox = widgets.VBox([self._wtab, self._wbutton])
    else:
      self._vbox = widgets.VBox([self._wtab])

    self._tabs = []
    self._tabdata = []
    self._uidata = None

    self.addTab()

  def addTab (self, *args, **kwargs):
    intflist = getInterfaces(self._data["manifest"])
    cols = []
    cols.append(IT.DropdownColumn("Interface", values = intflist))

    iptbl = IT.InputTable(cols, {"index" : len(self._tabs)}, None)
    tw = widgets.Text(description = "Subnet:")
    self._tabdata.append((tw, iptbl))
    tbox = widgets.VBox([tw, iptbl.display()])
    self._tabs.append(tbox)
    self._wtab.children = self._tabs

    for n in range(len(self._tabs)):
      self._wtab.set_title(n, str(n+1))

  def display (self):
    return self._vbox

  def getAnswerData (self):
    data = [{"_uidata" : self._uidata}]
    for subnetw, iptbl in self._tabdata:
      obj = {"subnet" : subnetw.value }
      obj["table"] = iptbl.getAnswerData()
      data.append(obj)
    return data
