# Copyright (c) 2019-2020  University of Houston Networking Lab

from IPython.core.getipython import get_ipython

from geni.support import ipython as gipy
from geni.aggregate.vts import VTS

# We get here second, so usually we want to replace the already-replaced value and preserve the
# original moved function (since we need to call it internally).
def replaceSymbol (module, name, func, move = False):
  """Moves module.name to module._name and sets module.name to the new function object."""
  if move:
    setattr(module, "_%s" % (name), getattr(module, name))
  setattr(module, name, func)

class FilterableRetListProxy(object):
  def __init__ (self, obj, columns, row_template, tupl = False, cfilter = None):
    self._obj = obj
    self._columns = columns
    self._row_template = row_template
    self._col_template = "<th>%s</th>"
    self._tuple = tupl

  def __len__ (self):
    return len(self._obj)

  def __getitem__ (self, i):
    return self._obj[i]

  def __getslice__ (self, i, j):
    i = max(i, 0)
    j = max(j, 0)
    return self._obj[i:j]

  def __contains__ (self, item):
    return item in self._obj

  def __iter__ (self):
    for x in self._obj:
      yield x

  def _repr_html_ (self):
    trlist = []
    for row in self._obj:
      if self._tuple:
        trlist.append(self._row_template.format(*row))
      else:
        trlist.append(self._row_template.format(**row))

    collist = []
    for column in self._columns:
      collist.append(self._col_template % (column))

    return """<table>\n<tr>%s</tr>\n%s\n<table""" % ("".join(collist), "\n".join(trlist))


PINFOCOLS = ["Client ID", "Port #", "ifindex", "PVID", "MTU", "Admin State", "Link State", "RX Bytes (Pkts)", "TX Bytes (Pkts)"]
PINFOROW = "<tr><td>{client-id}</td><td>{num}</td><td>{ifindex}</td><td>{tag}</td><td>{mtu}</td><td>{admin_state}</td><td>{link_state}</td><td>{statistics[rx_bytes]} ({statistics[rx_packets]})</td><td>{statistics[tx_bytes]} ({statistics[tx_packets]})</td></tr>"

PINFO_TABLE = [("Client ID", "client-id"),
               ("Port #", "num"),
               ("ifindex", "ifindex"),
               ("vlan", "tag"),
               ("MTU", "mtu"),
               ("Admin State", "admin_state"),
               ("Link State", "link_state"),
               ("RX Bytes (Pkts)", "{statistics[rx_bytes]} ({statistics[rx_packets]})"),
               ("TX Bytes (Pkts)", "{statistics[tx_bytes]} ({statistics[tx_packets]})")]

def getPortInfo (self, context, sname = None, client_ids = None):
  if sname is None:
    _ip = get_ipython()
    client_ids = context
    context = _ip.user_ns["context"]
    sname = _ip.user_ns["SLICE"]

  res = self._getPortInfo(context, sname, client_ids)

  if len(res) == 1:
    values = list(res.values())
    return gipy.RetListProxy(values[0], PINFOCOLS, PINFOROW)

  retobj = {}
  for k,v in res.items():
    retobj[k] = gipy.RetListProxy(v, PINFOCOLS, PINFOROW)
  return retobj

replaceSymbol(VTS, "getPortInfo", getPortInfo)

