# Copyright (c) 2021  University of Houston

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import getpass

from IPython.core.magic import Magics, magics_class, line_magic
import requests

import uhed.net.dashboard
from uhed.log import Logger

@magics_class
class InstructorMagics(Magics):
  def __init__ (self, shell):
    super(InstructorMagics, self).__init__(shell)
    self.uhm = shell.user_ns["_uhm"]

  @line_magic
  def browse (self, line):
    width = 3
    parts = line.strip().split()
    try:
      width = int(parts[1])
    except:
      pass
      
    return uhed.net.dashboard.UserDashboard(self.uhm, width).display()


def load_ipython_extension (ipy):
  try:
    uhm = ipy.user_ns["_uhm"]
  except KeyError:
    print("You must load the uhed extension before using this extension.")
    raise

  user = getpass.getuser()
  r = requests.get("%s/course/name/%s/instructor/%s" % (uhm._lsurl, uhm._course, uhm.context.userurn))
  if not r.json()["value"]:
    Logger.log("Attempt to load instructor extension by non-instructor (%s, %s)" % (user, uhm.context.userurn))
    print("You must be an instructor to use this extension.  Your action has been logged.")
    return

  ipy.register_magics(InstructorMagics)
  Logger.log("Instructor extension loaded by %s" % (user))
