# Copyright (c) 2017-2021  University of Houston Networking Lab

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

import uuid

import dateutil.parser

from .datafile import DataFile

TIMEFMT = "%Y-%m-%d %H:%M"

class Module(DataFile):
  EXTENSION = '.lmod'

  def __init__ (self):
    super(Module, self).__init__()
    self.uuid = str(uuid.uuid4())
    self.uiname = None
    self.params = None
    self.vertex_filters = {}
    self.site_filters = {}
    self.request = None
    self.course = None
    self.sort = None
    self.version = None
    self.save_manifest = False
    self.var_pool = []
    self.post_build = None
    self.pre_build = None
    self.topo_name = None
    self.instr_only = False
    self.exercise_only = False
    self.hard_start = None
    self.slice_suffix = None
    self.depends = []

  def __json__ (self):
    obj = {}
    obj["version"] = 8
    obj["uuid"] = self.uuid
    obj["uiname"] = self.uiname
    obj["request"] = self.request
    obj["course"] = self.course
    obj["sort"] = self.sort
    obj["params"] = self.params
    obj["vertex-filters"] = self.vertex_filters
    obj["site-filters"] = self.site_filters
    obj["var-pool"] = self.var_pool
    obj["save-manifest"] = self.save_manifest
    obj["post-build"] = self.post_build
    obj["pre-build"] = self.pre_build
    obj["topo-name"] = self.topo_name
    obj["instructor-only"] = self.instr_only
    obj["exercise-only"] = self.exercise_only
    obj["hard-start"] = self.hard_start
    obj["slice-suffix"] = self.slice_suffix
    obj["depends"] = self.depends

    if self.hard_start:
      obj["hard-start"] = obj["hard-start"].strftime(TIMEFMT)
    return obj

  @classmethod
  def jconstruct (cls, obj):
    mod = cls()
    if obj["version"] >= 1:
      mod.uiname = obj["uiname"]
      mod.request = obj["request"]
      mod.course = obj["course"]
      mod.sort = obj["sort"]

      # Modules won't have a UUID on first load
      if "uuid" in obj:
        mod.uuid = obj["uuid"]

      # These will need real loaders
      mod.params = obj["params"]
      mod.vertex_filters = obj["vertex-filters"]
      mod.site_filters = obj["site-filters"]

    if obj["version"] >= 2:
      mod.var_pool = obj["var-pool"]
      mod.save_manifest = obj["save-manifest"]
      mod.post_build = obj["post-build"]

    if obj["version"] >= 3:
      mod.topo_name = obj["topo-name"]

    if obj["version"] >= 4:
      mod.pre_build = obj["pre-build"]

    if obj["version"] >= 5:
      mod.instr_only = obj["instructor-only"]

    if obj["version"] >= 6:
      mod.exercise_only = obj["exercise-only"]

    if obj["version"] >= 7:
      if "hard-start" in obj and obj["hard-start"]:
        mod.hard_start = dateutil.parser.parse(obj["hard-start"])

    if obj["version"] >= 8:
      mod.slice_suffix = obj["slice-suffix"]
      mod.depends = obj["depends"]

    return mod
