# Copyright (c) 2021  University of Houston Networking Lab

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os.path

from .. import json
from ..log import KeyAdapter

class TeamsDB(object):
  def __init__ (self, labsrv_db, logger):
    self._log = KeyAdapter("TeamsDB", logger)
    self._ldb = labsrv_db
    self._cfg = self._ldb._cfg
    self._course_map = "%s/coursedata.msteams" % (self._ldb._root)
    self._user_map = "%s/userdata.msteams" % (self._ldb._root)
    self._exercise_map = "%s/exercisedata.msteams" % (self._ldb._root)
    
    self._courses = {}
    self._users = {}
    self._exercises = {}

    self._load()

  def _writeCourses (self):
    with open(self._course_map, "w+") as cmf:
      cmf.write(json.dumps(self._courses))

  def _writeUsers (self):
    with open(self._user_map, "w+") as umf:
      umf.write(json.dumps(self._users))

  def _writeExercises (self):
    with open(self._exercise_map, "w+") as emf:
      emf.write(json.dumps(self._exercises))

  def _load (self):
    if os.path.exists(self._course_map):
      self._courses = json.loads(open(self._course_map, "r").read())
    if os.path.exists(self._user_map):
      self._users = json.loads(open(self._user_map, "r").read())
    if os.path.exists(self._exercise_map):
      self._exercises = json.loads(open(self._exercise_map, "r").read())

  def setCourseAssignment (self, cname, euuid, asuuid):
    self._exercises.setdefault(cname, {})[euuid] = asuuid
    self._writeExercises()

  def getCourseAssignment (self, cname, euuid):
    return self._exercises[cname][euuid]

  def setCourseUUID (self, cname, uuidstr):
    self._courses[cname] = uuidstr
    self._log.debug("Setting UUID for (%s) to (%s)" % (cname, uuidstr))
    self._writeCourses()

  def getCourseUUID (self, cname):
    try:
      return self._courses[cname]
    except KeyError:
      self._log.error("Attempt to get UUID for unknown class (%s)" % (cname))
      return None

