# Copyright (c) 2016-2018  Barnstormer Softworks, Ltd.

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

import functools

from flask import url_for, redirect

from .json import jsonify_ephemeral
from .log import KeyAdapter

def route (path, methods):
  def decorator (target):
    target.route = ((path, methods))
    return target
  return decorator

class Dispatcher(object):
  def __init__ (self, key, logger, app):
    self._log = KeyAdapter(key, logger)
    self._app = app

    for k in dir(self):
      try:
        val = getattr(self, k)
        (path, methods) = val.route
      except TypeError:
        continue
      except AttributeError:
        continue

      try:
        app.add_url_rule(path, path, excwrapper(self._log, val), methods=methods)
        self._log.debug("Added route for: %s with method <%s>" % (path, k))
      except Exception as e:
        self._log.exception("Adding URL Rule")
        continue

    self._log.info("Loaded")

def excwrapper (log, func):
  @functools.wraps(func)
  def wrap (*args, **kwargs):
    try:
      return func(*args, **kwargs)
    except Exception as e:
      log.exception("Exception")
      return jsonify_ephemeral(e, -1)
  return wrap
